require "spec_helper"

describe Moe::TableManager do
  let(:dyna)    { Moe::Dyna.new }
  let(:manager) { Moe::TableManager.new }

  describe "#initialize" do
    it "creates itself a table if one does not exist" do
      new_manager = Moe::TableManager.new

      expect(
        dyna.find(new_manager.meta_table_names.first).table.table_name
      ).to eq(new_manager.meta_table_names.first)
    end
  end

  describe "#build" do
    it "creates a table for a given model" do
      manager.build "build_test"

      expect(
        dyna.find(manager.table_name "build_test_1").table.table_name
      ).to match("build_test")
    end

    it "creates additional tables if requested" do
      manager.build "mirror_test", 2

      expect(
        dyna.find("#{manager.table_name('mirror_test')}_2").table.table_name
      ).to match("mirror_test_2")
    end

    it "does not create additional tables by default" do
      manager.build "false_mirror_test"

      expect(
        dyna.find("#{manager.table_name('false_mirror_test')}_2")
      ).to be_false
    end

    it "munges model names into a DynamoDB-approved format" do
      manager.build "Testy::Model"

      expect(
        dyna.find(manager.table_name "testy_model_1").table.table_name
      ).to match("testy_model")
    end
  end

  describe "#increment" do
    it "increments the table" do
      manager.build "increment_test"

      Timecop.freeze(Date.today + 30) do
        frozen_manager = Moe::TableManager.new

        frozen_manager.increment "increment_test"
      end
    end

    it "pitches a fit if run twice on the same day" do
      manager.build "fit_test"

      expect { manager.increment "fit_test" }.to raise_error
    end
  end

  describe "#load_metadata" do
    it "loads metadata for a model" do
      metadata = { read_tables:  ["load_metadata_test"],
                   write_tables: ["load_metadata_test"] }

      manager.update_metadata "load_metadata_test", metadata

      expect(
        manager.load_metadata("load_metadata_test")[:write_tables].first
      ).to match("load_metadata_test")
    end
  end

  describe "#meta_table_name" do
    it "includes the RAILS_ENV if one exists" do
      ENV["RAILS_ENV"] = "test"

      expect( manager.meta_table_name ).to match("test_manager")
    end

    it "does not mind if there is no RAILS_ENV" do
      ENV["RAILS_ENV"] = ""

      expect( manager.meta_table_name ).to match("manager")
    end
  end

  describe "#table_name" do
    it "includes the RAILS_ENV if one exists" do
      ENV["RAILS_ENV"] = "test"

      expect( manager.table_name("tetsuo") ).to match("_test_")
    end

    it "does not mind if there is no RAILS_ENV" do
      ENV["RAILS_ENV"] = ""

      expect( manager.table_name("tetsuo") ).to match("tetsuo")
    end

    it "includes the date" do
      expect( manager.table_name("tetsuo") ).to match(manager.date)
    end
  end

  describe "#update_metadata" do
    it "updates the metadata" do
      manager.update_metadata "testie", { read_tables:  ["testie"],
                                          write_tables: ["testie"] }

      result = dyna.get_item manager.meta_table_names,
                                   { "hash" => { s: "testie" } }

      expect(
        result["payload"]["s"]
      ).to match("testie")
    end
  end
end
